(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[4502],{22814:function(e,i,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/notifications/digests/[id]",function(){return t(31363)}])},31363:function(e,i,t){"use strict";t.r(i);var n=t(24246),s=t(39158),d=t(86677),r=t(77213),a=t(77830),o=t(58754),u=t(19904),_=t(4496),c=t(40822),l=t(60240);i.default=()=>{let{id:e}=(0,d.useRouter)().query,{data:i,isLoading:t}=(0,c.nZ)({config_id:e},{skip:!e});return t?(0,n.jsx)(r.Z,{title:"Edit digest",children:(0,n.jsx)(s.xuv,{className:"flex justify-center py-12",children:(0,n.jsx)(s.$jN,{})})}):(0,n.jsx)(r.Z,{title:"Edit digest",children:(0,n.jsx)(u.ZP,{scopes:[l.Sh.DIGEST_CONFIG_CREATE,l.Sh.DIGEST_CONFIG_UPDATE],children:(0,n.jsxs)(s.xuv,{"data-testid":"edit-digest-config",children:[(0,n.jsx)(o.Z,{heading:(null==i?void 0:i.name)||"Edit Digest Configuration",breadcrumbItems:[{title:"Digests",href:a.DN},{title:(null==i?void 0:i.name)||"Edit"}]}),(0,n.jsx)(_.Z,{initialValues:i?{id:i.id,name:i.name,digest_type:i.digest_type,enabled:i.enabled,messaging_service_type:i.messaging_service_type,cron_expression:i.cron_expression,timezone:i.timezone}:void 0,isLoading:t})]})})})}}},function(e){e.O(0,[8212,4496,2888,9774,179],function(){return e(e.s=22814)}),_N_E=e.O()}]);