(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[3524],{98763:function(e,t,s){(window.__NEXT_P=window.__NEXT_P||[]).push(["/notifications/digests/new",function(){return s(48799)}])},48799:function(e,t,s){"use strict";s.r(t);var i=s(24246),n=s(39158),r=s(77213),c=s(77830),a=s(58754),u=s(19904),d=s(4496),l=s(60240);let{Text:o}=n.AntTypography;t.default=()=>(0,i.jsx)(r.Z,{title:"Create digest",children:(0,i.jsx)(u.ZP,{scopes:[l.Sh.DIGEST_CONFIG_CREATE,l.Sh.DIGEST_CONFIG_UPDATE],children:(0,i.jsxs)(n.xuv,{"data-testid":"new-digest-config",children:[(0,i.jsx)(a.Z,{heading:"Create digest",breadcrumbItems:[{title:"Digests",href:c.DN},{title:"New"}]}),(0,i.jsx)(o,{className:"mb-6 block text-sm",children:"Configure a new digest to receive email summaries on a regular schedule."}),(0,i.jsx)(d.Z,{})]})})})}},function(e){e.O(0,[8212,4496,2888,9774,179],function(){return e(e.s=98763)}),_N_E=e.O()}]);