"""adds optional duration field to assets

Revision ID: f108fa05c579
Revises: 9caf76161e55
Create Date: 2025-08-22 12:47:31.374493

"""

import sqlalchemy as sa
from alembic import op

# revision identifiers, used by Alembic.
revision = "f108fa05c579"
down_revision = "9caf76161e55"
branch_labels = None
depends_on = None


def upgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.add_column("asset", sa.Column("duration", sa.String(), nullable=True))
    # ### end Alembic commands ###


def downgrade():
    # ### commands auto generated by Alembic - please adjust! ###
    op.drop_column("asset", "duration")
    # ### end Alembic commands ###
