(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[7778],{96406:function(e,n,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/data-discovery/action-center/website",function(){return t(43340)}])},43340:function(e,n,t){"use strict";t.r(n);var u=t(86677),c=t(27378),r=t(77830);n.default=()=>{let e=(0,u.useRouter)();return(0,c.useLayoutEffect)(()=>{e.replace(r.vi)},[e]),null}}},function(e){e.O(0,[2888,9774,179],function(){return e(e.s=96406)}),_N_E=e.O()}]);