(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[624],{11124:function(e,n,t){(window.__NEXT_P=window.__NEXT_P||[]).push(["/data-discovery/action-center/datastore",function(){return t(29563)}])},29563:function(e,n,t){"use strict";t.r(n);var u=t(86677),r=t(27378),c=t(77830);n.default=()=>{let e=(0,u.useRouter)();return(0,r.useLayoutEffect)(()=>{e.replace(c.vi)},[e]),null}}},function(e){e.O(0,[2888,9774,179],function(){return e(e.s=11124)}),_N_E=e.O()}]);