<html>
  <head>
    <link
      rel="stylesheet"
      href="https://fonts.googleapis.com/css2?family=Inter:wght@400;500;600"
    />
    <link rel="stylesheet" href="../data/main.css" />
  </head>
  <body>
    <div class="container">
      <div class="header"></div>
      <div class="content">
        <div class="button-container">
          <a href="../clickme.html">
            <div class="button"><img src="../data/back.svg" /></div>
            <span>Back to main page</span>
          </a>
        </div>
        <h1>Attachments</h1>
        {% if not enable_streaming %}
        <p class="expiration-notice">
          Note: All download links will expire in {{ download_link_ttl_days }}
          days.
        </p>
        {% endif %}
        <div class="table table-hover">
          <div class="table-row">
            <div class="table-cell" style="text-align: left">File Name</div>
            <div class="table-cell" style="text-align: left">Size</div>
          </div>
          {% for name, info in data.items() %}
          <a href="{{ info.url }}" class="table-row" target="_blank">
            <div class="table-cell" style="text-align: left">
              {% if enable_streaming %}{{ name }}{% else %}{{ info.original_name
              }}{% endif %}
            </div>
            <div class="table-cell" style="text-align: left">
              {{ info.size }}
            </div>
          </a>
          {% endfor %}
        </div>
      </div>
    </div>
  </body>
</html>
