<!DOCTYPE html><html lang="en"><head><meta charSet="utf-8"/><meta name="viewport" content="width=device-width"/><meta name="next-head-count" content="2"/><link data-next-font="" rel="preconnect" href="/" crossorigin="anonymous"/><link rel="preload" href="/_next/static/css/a1e4be9466578ef1.css" as="style"/><link rel="stylesheet" href="/_next/static/css/a1e4be9466578ef1.css" data-n-g=""/><noscript data-n-css=""></noscript><script defer="" nomodule="" src="/_next/static/chunks/polyfills-42372ed130431b0a.js"></script><script src="/_next/static/chunks/webpack-09ca52bc7beb0b43.js" defer=""></script><script src="/_next/static/chunks/framework-c92fc3344e6fd165.js" defer=""></script><script src="/_next/static/chunks/main-090643377c8254e6.js" defer=""></script><script src="/_next/static/chunks/pages/_app-eb6669fa97f8eeb2.js" defer=""></script><script src="/_next/static/chunks/7630-7f75ab7b8df42eb3.js" defer=""></script><script src="/_next/static/chunks/pages/notifications/templates/add-template-c79e7724e4bc3899.js" defer=""></script><script src="/_next/static/IMBLbjII-szXDLei-TlXS/_buildManifest.js" defer=""></script><script src="/_next/static/IMBLbjII-szXDLei-TlXS/_ssgManifest.js" defer=""></script><style>.data-ant-cssinjs-cache-path{content:"";}</style></head><body><div id="__next"><div style="height:100%;display:flex"></div></div><script id="__NEXT_DATA__" type="application/json">{"props":{"pageProps":{}},"page":"/notifications/templates/add-template","query":{},"buildId":"IMBLbjII-szXDLei-TlXS","nextExport":true,"autoExport":true,"isFallback":false,"scriptLoader":[]}</script></body></html>