(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[9716],{27328:function(n,i,e){(window.__NEXT_P=window.__NEXT_P||[]).push(["/notifications/providers/new",function(){return e(46071)}])},46071:function(n,i,e){"use strict";e.r(i),e.d(i,{default:function(){return a}});var t=e(24246),r=e(39158),s=e(77213),o=e(90980),u=e(77830),f=e(10792);let c=()=>(0,t.jsxs)(s.Z,{title:"Create Messaging Configuration",children:[(0,t.jsx)(o.Z,{backPath:u.AD}),(0,t.jsx)(r.X6q,{fontSize:"2xl",fontWeight:"semibold",mb:6,children:"Configure your messaging provider"}),(0,t.jsx)(f.Z,{})]});var a=()=>(0,t.jsx)(c,{})}},function(n){n.O(0,[431,3662,7245,3377,2888,9774,179],function(){return n(n.s=27328)}),_N_E=n.O()}]);