(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[5970],{6920:function(e,l,a){(window.__NEXT_P=window.__NEXT_P||[]).push(["/poc/form-experiments/AntForm",function(){return a(33396)}])},95813:function(e,l,a){"use strict";a.d(l,{S:function(){return n}});let n={input:"default text",radio:"2",checkbox:!0,switch:!0,select:"2",multiselect:["2"],tags:["1"],date:"",number:12}},33396:function(e,l,a){"use strict";a.r(l),a.d(l,{AntFormPOC:function(){return u}});var n=a(24246),t=a(39158),s=a(3124),c=a.n(s),r=a(27378),i=a(95813);let{Title:o}=t.AntTypography,u=()=>{let[e]=t.PPS.useForm(),[l,a]=(0,r.useState)(i.S);return(0,n.jsxs)(t.bue,{children:[(0,n.jsxs)(t.JGx,{span:12,children:[(0,n.jsx)(o,{level:2,children:"Pure Ant"}),(0,n.jsxs)("div",{className:"mb-4",children:[(0,n.jsx)(t.j8w,{color:"success",children:"functional"}),(0,n.jsx)(t.j8w,{color:"success",children:"low effort"})]}),(0,n.jsxs)(t.PPS,{initialValues:{...i.S},form:e,layout:"vertical",onValuesChange:(e,l)=>{a(l)},children:[(0,n.jsx)(t.PPS.Item,{label:"Input",name:"input",children:(0,n.jsx)(t.uFc,{"aria-label":"Input"})}),(0,n.jsx)(t.PPS.Item,{name:"radio",children:(0,n.jsxs)(t.y02.Group,{name:"radio",children:[(0,n.jsx)(t.y02,{value:"1",children:"Radio 1"}),(0,n.jsx)(t.y02,{value:"2",children:"Radio 2"})]})}),(0,n.jsx)(t.PPS.Item,{name:"checkbox",valuePropName:"checked",children:(0,n.jsx)(t.E_O,{children:"Checkbox"})}),(0,n.jsx)(t.PPS.Item,{label:"Switch",name:"switch",valuePropName:"checked",children:(0,n.jsx)(t.rAg,{})}),(0,n.jsx)(t.PPS.Item,{label:"Select",name:"select",children:(0,n.jsx)(t.WPr,{allowClear:!0,className:"w-full",options:[{value:"1",label:"Option 1"},{value:"2",label:"Option 2"}],"aria-label":"Select"})}),(0,n.jsx)(t.PPS.Item,{label:"Multiselect",name:"multiselect",children:(0,n.jsx)(t.WPr,{mode:"multiple",allowClear:!0,className:"w-full",options:[{value:"1",label:"Option 1"},{value:"2",label:"Option 2"}],"aria-label":"Select"})}),(0,n.jsx)(t.PPS.Item,{label:"Tags",name:"tags",children:(0,n.jsx)(t.WPr,{mode:"tags",allowClear:!0,className:"w-full",options:[{value:"1",label:"Option 1"},{value:"2",label:"Option 2"}],"aria-label":"Select"})}),(0,n.jsx)(t.PPS.Item,{label:"Date",name:"date",children:(0,n.jsx)(t.cv2,{})}),(0,n.jsx)(t.PPS.Item,{label:"Number",name:"number",children:(0,n.jsx)(t.W7D,{"aria-label":"InputNumber"})})]})]}),(0,n.jsxs)(t.JGx,{span:8,offset:4,children:[(0,n.jsx)(o,{level:4,children:"Controlled Values"}),(0,n.jsx)(t.V4A,{style:{backgroundColor:c().FIDESUI_MINOS,color:c().FIDESUI_CORINTH},children:(0,n.jsx)("pre",{children:(0,n.jsxs)("code",{children:["{\n",Object.keys(l).map(e=>"  ".concat(e.toString(),": ").concat(l[e],"\n")),"}"]})})})]})]})};l.default=u}},function(e){e.O(0,[2888,9774,179],function(){return e(e.s=6920)}),_N_E=e.O()}]);