.ConnectionTypeLogo_connectionTypeLogo__dOKpm{background-color:transparent;color:var(--ant-color-text)}.MonitorResult_monitor-result__title__Wqv_l{white-space:nowrap;font-weight:400}.MonitorResult_monitor-result__title__Wqv_l a{font-weight:600}