(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[4322],{74833:function(t,e,n){var i=n(56127),r=/^\s+/;t.exports=function(t){return t?t.slice(0,i(t)+1).replace(r,""):t}},56127:function(t){var e=/\s/;t.exports=function(t){for(var n=t.length;n--&&e.test(t.charAt(n)););return n}},66726:function(t,e,n){var i=n(11611),r=n(82846),o=n(91936),a=Math.max,l=Math.min;t.exports=function(t,e,n){var u,s,c,f,d,p,v=0,h=!1,x=!1,m=!0;if("function"!=typeof t)throw TypeError("Expected a function");function g(e){var n=u,i=s;return u=s=void 0,v=e,f=t.apply(i,n)}function b(t){var n=t-p,i=t-v;return void 0===p||n>=e||n<0||x&&i>=c}function j(){var t,n,i,o=r();if(b(o))return y(o);d=setTimeout(j,(t=o-p,n=o-v,i=e-t,x?l(i,c-n):i))}function y(t){return(d=void 0,m&&u)?g(t):(u=s=void 0,f)}function k(){var t,n=r(),i=b(n);if(u=arguments,s=this,p=n,i){if(void 0===d)return v=t=p,d=setTimeout(j,e),h?g(t):f;if(x)return clearTimeout(d),d=setTimeout(j,e),g(p)}return void 0===d&&(d=setTimeout(j,e)),f}return e=o(e)||0,i(n)&&(h=!!n.leading,c=(x="maxWait"in n)?a(o(n.maxWait)||0,e):c,m="trailing"in n?!!n.trailing:m),k.cancel=function(){void 0!==d&&clearTimeout(d),v=0,u=p=s=d=void 0},k.flush=function(){return void 0===d?f:y(r())},k}},82846:function(t,e,n){var i=n(77400);t.exports=function(){return i.Date.now()}},91936:function(t,e,n){var i=n(74833),r=n(11611),o=n(55193),a=0/0,l=/^[-+]0x[0-9a-f]+$/i,u=/^0b[01]+$/i,s=/^0o[0-7]+$/i,c=parseInt;t.exports=function(t){if("number"==typeof t)return t;if(o(t))return a;if(r(t)){var e="function"==typeof t.valueOf?t.valueOf():t;t=r(e)?e+"":e}if("string"!=typeof t)return 0===t?t:+t;t=i(t);var n=u.test(t);return n||s.test(t)?c(t.slice(2),n?2:8):l.test(t)?a:+t}},30952:function(t,e,n){"use strict";n.d(e,{f:function(){return u}});var i=n(24246),r=n(66726),o=n.n(r),a=n(27378),l=n(26917);let u=t=>{let{value:e,onChange:n,placeholder:r,...u}=t,[s,c]=(0,a.useState)(e);(0,a.useEffect)(()=>{c(e||"")},[e]);let f=o()(n,500),d=(0,a.useCallback)(t=>{c(t),f(t)},[]);return(0,i.jsx)(l.Z,{value:s,onChange:d,onClear:()=>{c(""),n("")},placeholder:r,...u})}},58754:function(t,e,n){"use strict";var i=n(24246),r=n(39158),o=n(70788);e.Z=t=>{let{heading:e,breadcrumbItems:n,isSticky:a=!0,children:l,rightContent:u,style:s,...c}=t;return(0,i.jsxs)("div",{...c,style:a?{position:"sticky",top:"-24px",paddingTop:"24px",paddingBottom:"24px",paddingLeft:"40px",marginLeft:"-40px",paddingRight:"40px",marginRight:"-40px",marginTop:"-24px",left:0,zIndex:20,backgroundColor:"white",...s}:{paddingBottom:"24px",...s},children:[(0,i.jsxs)(r.jqI,{justify:"space-between",children:["string"==typeof e?(0,i.jsx)(r.lQT,{className:n||l?"pb-4":void 0,level:1,"data-testid":"page-heading",children:e}):e,u&&(0,i.jsx)("div",{"data-testid":"page-header-right-content",children:u})]}),!!n&&(0,i.jsx)(o.m,{className:l?"pb-4":void 0,items:n,"data-testid":"page-breadcrumb"}),l]})}},70788:function(t,e,n){"use strict";n.d(e,{m:function(){return s}});var i=n(24246),r=n(39158),o=n(79894),a=n.n(o),l=n(27378);let{Text:u}=r.AntTypography,s=t=>{let{items:e,...n}=t,o=(0,l.useMemo)(()=>null==e?void 0:e.map((t,n)=>{let o=n===e.length-1,l={...t},s=l.onClick&&!l.href;return("string"==typeof l.title&&(l.title=(0,i.jsx)(u,{style:{color:"inherit",maxWidth:o?void 0:400},ellipsis:!o,id:o?"breadcrumb-current-page":void 0,children:l.title})),s)?l.title=(0,i.jsx)(r.wpx,{type:"text",size:"small",icon:l.icon,onClick:l.onClick,className:"ant-breadcrumb-link -mt-px px-1 text-inherit",children:l.title}):(l.icon&&(l.title=(0,i.jsxs)(i.Fragment,{children:[(0,i.jsx)("span",{className:"anticon align-text-bottom",children:l.icon}),l.title]})),l.href&&l.title&&(l.title=(0,i.jsx)(a(),{href:l.href,className:"ant-breadcrumb-link",children:l.title}),delete l.href)),l}),[e]);return(0,i.jsx)(r.zrq,{items:o,...n})}},79947:function(t,e,n){"use strict";var i=n(24246),r=n(39158);e.Z=t=>{let{type:e}=t;return(0,i.jsx)(r.kCb,{align:"center",h:"full",children:!!e&&(0,i.jsx)(r.j8w,{children:e})})}},7940:function(t,e,n){"use strict";n.d(e,{G:function(){return r}});var i=n(60240);let r=t=>{var e,n,r;return t?t.resource_type?t.resource_type:(null===(e=t.schemas)||void 0===e?void 0:e.length)?i.D$.DATABASE:(null===(n=t.tables)||void 0===n?void 0:n.length)?i.D$.SCHEMA:(null===(r=t.fields)||void 0===r?void 0:r.length)?i.D$.TABLE:i.D$.FIELD:void 0}},98559:function(t,e){"use strict";e.Z=t=>{let{name:e,urn:n,monitor_config_id:i,database_name:r,schema_name:o,table_name:a,top_level_field_name:l,top_level_field_urn:u}=t;if(!l)return e;let s=n.split(".");return u?n.replace("".concat(u).concat("."),""):([i,r,o,a,l].forEach(t=>{if(t){let e=s.indexOf(t);e>-1&&s.splice(e,1)}}),s.join("."))}},36168:function(t,e){"use strict";e.Z=t=>{var e;return!!t.parent_table_urn&&null!==(e=t.sub_field_urns)&&void 0!==e&&!!e.length&&!t.top_level_field_name}},80356:function(t,e,n){"use strict";var i=n(7940),r=n(36168),o=n(60240);e.Z=t=>(0,i.G)(t)!==o.D$.FIELD||(0,r.Z)(t)}}]);