.ConnectionTypeLogo_connectionTypeLogo__dOKpm{background-color:transparent;color:var(--ant-color-text)}.TaxonomySelect_option__vY6v2 strong{font-weight:400}.TaxonomySelect_option__vY6v2 em{visibility:hidden}.TaxonomySelect_option__vY6v2:hover strong{font-weight:600}.TaxonomySelect_option__vY6v2:hover em{visibility:visible}.MonitorFieldListItem_monitor-field__title__YrxOU{white-space:nowrap;font-weight:400}.MonitorFieldListItem_monitor-field__title__YrxOU button{font-weight:600}.MonitorFieldListItem_monitor-field__breadcrumb__Rn4vE ol{overflow:hidden;flex-wrap:nowrap}.MonitorFieldListItem_monitor-field__breadcrumb__Rn4vE li:last-child{color:var(--ant-color-text-secondary)}.MonitorTree_monitor-tree__QKz6T .ant-tree-node-content-wrapper{padding-right:0;padding-left:0}.MonitorTree_monitor-tree__QKz6T .ant-tree-icon__customize{flex:none}