.ConnectionGrid_grid-row__Cv_Zf{border-top-width:.5px;border-color:var(--chakra-colors-blackAlpha-300);box-sizing:border-box}.ConnectionGrid_grid-item__xP7eK{border-right-width:.5px;border-color:var(--chakra-colors-blackAlpha-300);box-sizing:border-box}.ConnectionGrid_grid-item__xP7eK:nth-child(3n){border-right-width:0}.ConnectionTypeLogo_connectionTypeLogo__dOKpm{background-color:transparent;color:var(--ant-color-text)}