(self.webpackChunk_N_E=self.webpackChunk_N_E||[]).push([[8043],{52507:function(n,u,e){(window.__NEXT_P=window.__NEXT_P||[]).push(["/data-discovery/action-center/infrastructure",function(){return e(72245)}])},72245:function(n,u,e){"use strict";e.r(u);var t=e(86677),r=e(27378),c=e(77830);u.default=()=>{let n=(0,t.useRouter)();return(0,r.useLayoutEffect)(()=>{n.replace(c.vi)},[n]),null}}},function(n){n.O(0,[2888,9774,179],function(){return n(n.s=52507)}),_N_E=n.O()}]);