<html>
  <head>
    <link
      rel="stylesheet"
      href="https://fonts.googleapis.com/css2?family=Inter:wght@400;500;600"
    />
    <link rel="stylesheet" href="../main.css" />
  </head>
  <body>
    <div class="container">
      <div class="header"></div>
      <div class="content">
        <div class="button-container">
          <a href="../../clickme.html">
            <div class="button"><img src="../back.svg" /></div>
            <span>Back to datasets</span>
          </a>
        </div>
        <h1>{{ heading }}</h1>
        <div class="table table-hover">
          <div class="table-row">
            <div class="table-cell" style="text-align: left">Collection</div>
          </div>
          {% for name, link in data.items() %}
          <a href="{{ link }}" class="table-row">
            <div class="table-cell" style="text-align: left">{{ name }}</div>
          </a>
          {% endfor %}
        </div>
      </div>
    </div>
  </body>
</html>
